%% Copyright (C) 2003 by Institute of Scientific Computing.
%% Technical University Braunschweig, All rights reserved.
%% 
%% 
%% This file is part of the wire_article package
%% 
%%                Internet:  <wire@tu-bs.de>
%% $Id: wire_article.cls,v 1.1 2007/01/19 07:15:22 krosche Exp $

\def\filedate{2002/11/11}
\def\fileversion{1.0}
\def\filename{wire_article.cls}
\NeedsTeXFormat{LaTeX2e}

\ProvidesClass{wire_article}[\filedate\space v\fileversion\space
  wire_article class]


\DeclareOption{a4paper}
   {\setlength\paperheight {297mm}%
    \setlength\paperwidth  {210mm}}

\def\@myptsize{0}
\DeclareOption{10pt}{\renewcommand\@myptsize{0}}
\DeclareOption{11pt}{\renewcommand\@myptsize{1}}
\DeclareOption{12pt}{\renewcommand\@myptsize{2}}

\def\@wireside{oneside}
\DeclareOption{oneside}{\renewcommand\@wireside{oneside}}
\DeclareOption{twoside}{\renewcommand\@wireside{twoside}}

\newif\ifgerman
\DeclareOption{german}{\germantrue}
\DeclareOption{english}{\germanfalse}


\def\@contri{\ }
\def\@versi{\ }

\def\InformatikberichtNr#1{\gdef\@InformatikberichtNr{#1}}
\def\authorname#1{\gdef\@authorname{#1}}

\def\contributedTo#1{\renewcommand\@contri{\noindent #1 }}
\def\versionOf#1{\renewcommand\@versi{\noindent #1 }}


\ExecuteOptions{a4paper, 12pt, oneside, en} 
\ProcessOptions

\LoadClass[1\@myptsize pt, a4paper, \@wireside]{article}

\ifgerman
\def\telefontext{{\footnotesize Telefon\/:}}
\def\telefaxtext{{\footnotesize Fax\/:}}
\def\emailtext{{\footnotesize E--Mail\/:}}
\def\httptext{{\footnotesize www\/:}}
\def\wire{Institut f\"ur Wissenschaftliches Rechnen}
\def\tubs{Technische Universit\"at Braunschweig}
\def\street{Hans-Sommer-Strasse 65}
\def\plzort{D--38106 Braunschweig, Germany}
\def\tubstitle{Technische Universit\"at Braunschweig\\
               Braunschweig, Germany}
\def\fb1{Fachbereich f\"ur Mathematik und Informatik}
\def\contacttxt{\textbf{Kontakt}}
\def\addresstxt{\textbf{Adresse}}
\def\posttxt{\textbf{Anschrift}}

\else % english
\def\telefontext{{\footnotesize Phone\/:}}
\def\telefaxtext{{\footnotesize Fax\/:}}
\def\emailtext{{\footnotesize E--Mail\/:}}
\def\httptext{{\footnotesize www\/:}}
\def\wire{Institute of Scientific Computing}
\def\tubs{Technical University Braunschweig}
\def\street{Hans-Sommer-Strasse 65}
\def\plzort{D--38106 Braunschweig, Germany}
\def\tubstitle{Technical University Braunschweig\\
               Brunswick, Germany}
\def\fb1{Department of Mathematics and Computer Science}
\def\contacttxt{\textbf{Contact}}
\def\addresstxt{\textbf{Location}}
\def\posttxt{\textbf{Postal Address}}

\fi %\ifgerman

\def\phone{+49-(0)531-391-3000}
\def\fax{+49-(0)531-391-3003}
\def\email{\tt{wire@tu-bs.de}}
\def\www{\tt{http://www.tu-bs.de/institute/WiR}}

\def\tubsPLZ{D-38092 Braunschweig}
\def\wirePLZ{D-38106 Braunschweig}

\def\wirede{Institut f\"ur Wissenschaftliches Rechnen}
\def\tubsde{Technische Universit\"at Braunschweig}
\def\informatikbericht#1{Informatikbericht Nr.: #1}


\renewcommand{\maketitle}{
  \begin{titlepage}
    \begin{center}
      \includegraphics[scale=0.5]{sc-logo}
    \end{center}

    \vspace{0.7cm}
    \begin{center}
      \Large {\bf \@title}
    \end{center}

    \vspace{0.3cm}
    \begin{center}
      \@authorname \\
      \wire \\
      \tubstitle
    \end{center}

    \vspace{0.7cm}
    \begin{center}
      \informatikbericht{\@InformatikberichtNr}\\[0.2cm]
      \@date
    \end{center}
    
    \vspace{0.7cm}
    \begin{center}
      \includegraphics[scale=0.2]{logo_tubs}
    \end{center}

  \end{titlepage}

  \newpage
  % second page
  \thispagestyle{empty}
  \begin{center}
  \Large{\@title} \\[1cm]
  \normalsize \@authorname \\

  \fb1 \\
  \tubstitle\\[1.0cm]
  \informatikbericht{\@InformatikberichtNr}\\[0.2cm]
  \@date   \\ [1cm]
  \@contri \\ [0.7cm]
  \@versi  \\ [0.7cm]

  \end{center}

  \vfill
%------------------------------------
  \begin{footnotesize}
    \begin{tabular}{ll}
      \addresstxt & \posttxt\\
      \wire & \wirede \\
      \tubs & \tubsde \\
      \street & \tubsPLZ \\
      \wirePLZ & Germany\\
    \end{tabular}\\ [4pt]

    \begin{tabular}{rl}
      \contacttxt & \\
      \telefontext & \phone\\
      \telefaxtext & \fax\\
      \emailtext & \email\\
      \httptext  & \www
    \end{tabular} \\ [20pt]

%    \begin{tabular}{ll}
%      \textbf{Copyright} & \\
%      \copyright by Institut f\"ur Wissenschaftliches Rechnen,
%      Technische Universit\"at Braunschweig & \\
%    \end{tabular}

      {\noindent \textbf{Copyright}}
      \copyright\hspace{0.04cm} by Institut f\"ur Wissenschaftliches Rechnen,
      Technische Universit\"at Braunschweig

      {\noindent
      This work is subject to copyright. All rights are reserved,
      whether the whole or part of the material is concerned,
      specifically the rights of translation, reprinting, reuse of
      illustrations, recitation, broadcasting, reproduction on
      microfilm or in any other way, and storage in data banks. Duplication
      of this publication or parts thereof is permitted in connection with
      reviews or scholarly analysis. Permission for use must always be
      obtained from the copyright holder.}\\

      {\noindent
      Alle Rechte vorbehalten, auch das des auszugsweisen Nachdrucks, der
      auszugsweisen oder vollst\"andigen Wiedergabe
      (Photographie, Mikroskopie), der Speicherung in
       Datenverarbeitungsanlagen und das der \"Ubersetzung.}


  \end{footnotesize}

%------------------------------------

  \newpage
  % here starts the paper ...
  \setcounter{page}{1}
  \begin{center}
    \Large {\@title} \\[1cm]
    \normalsize \@author
    \@thanks \  \\ \@date
  \end{center}
}

\endinput
%% End of file `wire_article.cls'.

%% ------------------ CVS Log entries ---------------------
%% $Log: wire_article.cls,v $
%% Revision 1.1  2007/01/19 07:15:22  krosche
%% Initial version
%%
%% Revision 1.2  2003/08/22 05:14:31  mkrosche
%% update the example ...
%%
%% Revision 1.2  2003/08/22 05:13:05  mkrosche
%% change the copyright section on the 'second page'
%%
%% Revision 1.1.1.1  2003/07/02 08:49:35  mkrosche
%% initial import 
%%
