package CTL.Serialize;

import CTL.*;
import CTL.Types.*;
import ReflWrap.*;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;

public class CTLIn extends SerialIn
{
	/** Generate a new stream
	 * @param in Underlying input stream
	 */
	public CTLIn (InputStream in) throws IOException
	{
		super(in);
	}

	/** Retrieve a string representation of this object
	 * @return String 
	 */
	public String toString ()
	{
		String ret = "";
		
		try
		{
			ret = hashCode()+" -- "+available();
		} catch (IOException e) {}

		return ret;
	}

	/** Read a C-style string from the stream (1 byte chars) 
	 * @return Value
	 */
	public String readString () throws IOException
	{
		CharArrayWriter ch = new CharArrayWriter();	
		char c=(char)readByte();
		
		while (c!=0)
		{   
			ch.write(c);
			c=(char)readByte();
		}

		return ch.toString();
	}

	/** Read a wide string from the stream (2 byte chars)
	 * @return Value
	 */
	public String readWString () throws IOException
	{
		CharArrayWriter ch = new CharArrayWriter();
		char c = readChar();
		
		while (c!=0)
		{
			ch.write(c);
			c = readChar();
		}

		return ch.toString();
	}
	
	/** Read an array from the stream
	 * @param type Type of the array
	 * @param params Type parameters
	 * @return Value
	 */
	public <T extends Object> T[] readArray (Class<T> type, 
		LinkedList<TypeTree> param) throws IOException, ClassNotFoundException,
		InstantiationException, IllegalAccessException, 
		InvocationTargetException
	{
		int len = (int)readLong();
		Env.log.msg(Logger.DBG, "Reading array with "+len+" elements.");
		return __readArray(type, param, len);
	}

	/** Read the elements of an array from the stream
	 * @param type Type of the array
	 * @param len Number of elements to read
	 * @return Value
	 */
	public <T extends Object> T[] __readArray (Class<T> type, 
		LinkedList<TypeTree> param, int len) throws IOException, 
		ClassNotFoundException, InstantiationException,	IllegalAccessException, 
		InvocationTargetException
	{
		if (len==0)
			return null;

		Method m = RUtil.serialrd2;
		Object[] args = new Object[1];
		args[0] = new TypeTree(type, param);
		
		T[] arr = (T[])new Object[len];
		for (int i=0;i<len;i++)
		{
			arr[i] = (T)m.invoke(this, args);
			if (arr[i] == null)
				Env.log.msg(Logger.ERR, "Arrary element is null.");
		}

		Env.log.msg(Logger.DBG2, "CTLIn.readArray(): "+Arrays.toString(arr));
		return arr;	
	}

	/** Read a tupel from the stream
	 * @param types Type parameters of the tupel
	 * @return Value
	 */
	public Tupel readTupel (Class[] types) throws IOException, 
		ClassNotFoundException, InstantiationException, IllegalAccessException,
		InvocationTargetException, CTLException
	{
		if (types==null)
			return null;
			
		Tupel ret = new Tupel(types);
		ret.read(this);
		return ret;
	}
	
	/** Read an object from the stream (Any-Object)
	 * @return Value
	 */
	public Object readObject () throws IOException, ClassNotFoundException,
		InstantiationException, IllegalAccessException,
		InvocationTargetException
	{
		String klass = readString();
		if (klass.equals(""))
			return null;
		long size = readLong();
		Class toread = Env.AnyObjIdent.get(klass);
		Env.log.msg(Logger.INFO, "Read an object of class "+toread.getName());
		return serialRead(toread);
	}
}
