package CTL.Serialize;

import CTL.*;
import CTL.Types.*;
import ReflWrap.*;

import java.io.*;
import java.lang.reflect.*;

public class CTLOut extends SerialOut
{
	/** Generate a new stream
	 * @param out Underlying output stream
	 */
	public CTLOut (OutputStream out) throws IOException
	{
		super(out);
	}

	/** Write a string to the stream (C-style one byte characters)
	 * @param data Value
	 */
	public void writeString (String data) throws IOException
	{
		if (data != null)
			writeBytes(data);
		writeByte(0);
	}

	/** Write a standard Java wide string (2 byte chars) to the stream
	 * @param data Value
	 */
	public void writeWString (String data) throws IOException
	{
		if (data != null)
			writeChars(data);
		writeChar(0);
	}
	
	/** Write an array to the stream
	 * @param data Value
	 */
	public <T extends Object> void writeArray (T data) throws IOException,
		IllegalAccessException, InvocationTargetException
	{
		if (data==null)
		{
			writeInt(0);
			return;
		}

		int len = java.lang.reflect.Array.getLength(data);
		writeLong(len);
		if (len > 0)
			__writeArray(data);
	}
		
	/** Write the array's elements to the stream
	 * @param data Value
	 */
	public <T extends Object> void __writeArray (T data) throws IOException, 
		IllegalAccessException, InvocationTargetException
	{
		if (data==null)
			return;

		String type = null;

		try
		{
			type = java.lang.reflect.Array.get(data, 0).getClass().getName();
		}
		catch (NullPointerException e)
		{
			type = Refl.arrayType(data.getClass().getName());
		}

		Method m = RUtil.getWriter(type);
		// TODO: Is this always correct?
		if (m.getName().equals("writeObject"))
			m = RUtil.serialwr;
		Object[] args = new Object[1];
		
		for (int i=0;i<java.lang.reflect.Array.getLength(data);i++)
		{
			args[0] = java.lang.reflect.Array.get(data, i);
			m.invoke(this, args);
		}
	}

	/** Write an object to the stream (Any-Object)
	 * @param data Value
	 */
	public void writeObject (Object data) throws IOException, 
		IllegalAccessException, InvocationTargetException, CTLException
	{
		Env.log.msg(Logger.DBG, "writeObject() called.");
		
		if (data == null)
		{
			Env.log.msg(Logger.DBG, "Payload is null.");
			writeString("");
			return;
		}

		String typeId = Env.AnyObjIdent.get(data.getClass());
		long size = RUtil.objSize(data);
		Env.log.msg(Logger.DBG, "Writing typeId "+typeId+
			" for object of class "+data.getClass().getName()+
			" with size "+size+".");
		writeString(typeId);
		writeLong(size);
		serialWrite(data);
	}

	protected void writeData (Object data) throws IOException, 
		IllegalAccessException, InvocationTargetException, CTLException
	{
		((Writable)data).write(this);
	}
}
