package CTL.Types;

import CTL.*;
import CTL.Comm.*;
import CTL.Serialize.*;

import java.io.*;
import java.lang.reflect.*;

/** Remote pointer */
public class rPointer extends Measure implements Writable
{
	/** Remote PeerID */
	private PeerID pid;
	/** ObjectID */
	private long objID;
	/** Reference count */
	private int refC;

	/** Checks if two objects are equal
	 * @param foo Object to compare to
	 * @return True if equal, false otherwise
	 */
	public boolean equals (Object foo)
	{
		if (!(foo instanceof rPointer))
			return false;
		if (toString().equals(foo.toString()))
			return true;
		return false;
	}
	
	/** Generate a new rPointer
	 * @param pid Remote pid
	 * @param objID ObjectID
	 */
	public rPointer (PeerID pid, long objID)
	{
		this.pid = pid;
		this.objID = objID;
		this.refC = 0;
	}

	/** Generate a new rPointer for an object stored on localhost
	 * @param objID ObjectID
	 * @param info Group information object
	 */
	public rPointer (long objID, GroupInfo info)
	{
		this((info == null) ? null : info.pid(), objID);
	}

	/** Retrieve a String representation of this object
	 * @return String
	 */
	public String toString ()
	{
		return objID+"@"+pid.host()+":"+pid.port()+", refCount: "+refC;
	}

	/** Retrieve the stored objID
	 * @return ObjectID
	 */
	public long objID ()
	{
		return objID;
	}

	/** Set a new ObjectID
	 * @param objID ObjectID
	 */
	public void setObjID (long objID)
	{
		this.objID = objID;
	}

	/** Retrieve the stored PeerID
	 * @return PeerID
	 */
	public PeerID peerID ()
	{
		return pid;
	}

	/** Serial read function
	 * @param in Input stream
	 */
	public void read (SerialIn in) throws IOException, ClassNotFoundException,
		InstantiationException, IllegalAccessException, InvocationTargetException
	{
		pid = in.serialRead(PeerID.class);
		objID = in.readLong();
		refC = in.readInt();
	}
	
	/** Serial write function
	 * @param out Output stream
	 */
	public void write (SerialOut out) throws IOException, IllegalAccessException, 
		InvocationTargetException, CTLException
	{
		out.serialWrite(pid);
		out.writeLong(objID);
		out.writeInt(refC);
	}
}
