package CodeGen;

import ReflWrap.*;

import java.io.*;
import java.util.*;

/** Code generator for the local wrapper */
public class GenLocal extends Gen
{
	private String implName = null;

	public GenLocal (String klass, HashMap<String,String> replaceMe, 
		String targetDir, String impl) throws ClassNotFoundException
	{
		super(klass, replaceMe, targetDir);
		name = classNamer(cinfo, "Local");
		supername = classNamer(cinfo, "CI");
		implName = impl;
	}

	protected void gen (OutputStreamWriter ostr) throws IOException, NoSuchMethodException
	{
		writeTo(ostr, genHead());
		writeTo(ostr, "private "+getFQCN()+" self = null;\n");
		writeTo(ostr, genConstructors());
		writeTo(ostr, genWrappers());
		writeTo(ostr, genAccept());
		writeTo(ostr, "}\n");
	}

	protected String getFQCN ()
	{
		String name = super.getFQCN();
		if (implName != null)
			return implName;
		return name;
	}

	protected String genConstructors () throws IOException
    {
        StringBuffer ret = new StringBuffer();
		
		if (constr.length == 0)
			ret.append("public "+name+ " ()\n{\nsuper((Object)null);\n}\n");

        for (Konstruktor con : constr)
        {
            String arglist = "";
            for (int j=0;j<con.len();j++)
                arglist += "arg"+j+((j==con.len()-1) ? "" : ", ");
			ret.append("public "+name+" ("+con.params()+"\n{\n"+
					((!supername.equals("RI")) ? "super((Object)null);\n" : "")+
                    "try\n{\n"+
                    "self = new "+getFQCN()+"("+arglist+");\n"+
                    // TODO: Add error handling.
                    "}\ncatch(Exception e)\n{}\n"+
                    "}\n");
		}

		return ret.toString();
	}
	
	protected String genWrappers () throws IOException, NoSuchMethodException
	{
		StringBuffer ret = new StringBuffer();

		for (int i=0;i<methods.length;i++)
		{
			Methode meth = methods[i];
			
			if ((meth == null) || (!meth.here()) || (meth.dont()))
				continue;
			
			for (int rres=0;rres<2;rres++)
			{
				boolean rr = rres==1;

				if (meth.isVoid() && rr)
					continue;
				
				String head = meth.head(rr);
				
				if (meth.isVoid())
				{
					if (meth.isStatic())
						ret.append(head+"\n{\n"+
							newMethod(cinfo.name(), meth.name(), meth.len(), false)+"}\n");
					else
					{
						ret.append(head+"\n{\n");
						ret.append("try\n{\n");
						ret.append(newMethod("self", meth.name(), meth.len(), false));
						ret.append("}\ncatch (Exception e)\n{\n");
						ret.append("e.printStackTrace();\n}\n");
						ret.append("}\n");
					}
				}
				else
				{
					if (rr)
					{
						ret.append(head+"\n{\nSystem.out.println(\"*_rr() "+
							"methods are not available for local objects.\");");
						ret.append("return null;\n}\n");
					}
					else
					{
						if (meth.isStatic())
							ret.append(head+"\n{\n"+
								newMethod(cinfo.name(), meth.name(), meth.len())+"}\n");
						else
						{
							// TODO: Handle exceptions properly
							ret.append(head+"\n{\n");
							ret.append("try\n{\n");
							ret.append(newMethod("self", meth.name(), meth.len()));
							ret.append("}\ncatch (Exception e) {}\n");
							ret.append("return "+Refl.defaultVal(meth.returns())+";\n");
							ret.append("}\n");
						}
					}
				}			
			}
		}

		return ret.toString();
	}
}
