package CodeGen;

import CTL.Annotate.*;
import ReflWrap.*;

import java.lang.reflect.*;
import java.util.*;

/** Helper class for keeping track of constructors inside CodeGen */
public class Konstruktor
{
	/** The reflection constructor */
	@SuppressWarnings("unused") private Constructor con;
	/** Info wrapper */
	private ConstructInfo cInfo;
	/** Parameters this constructor takes */
	private String params;
	/** Number of parameters it takes */
	private int len;

	/** Default construtor 
	 * @param con Reflection constructor object 
	 */
	public Konstruktor (Constructor con)
	{
		this.con = con;
		cInfo = new ConstructInfo(con);
		params = cInfo.paramsToString();
		len = cInfo.params().length;
	}

	// TODO: document
	public Konstruktor (String name)
	{
		con = null;
		cInfo = new ConstructInfo(name);
		params = ")";
		len = 0;
	}

	/** Parameters this constructor takes
	 * @return Parameters
	 */
	public String params ()
	{
		return params;
	}

	/** Number of parameters this constructor takes
	 * @return Number of parameters
	 */
	public int len ()
	{
		return len;
	}

	/** Generate Konstruktor-objects for all constructors in a class 
	 * @param cinfo Info object for the class
	 * @return Array of Konstruktor-objects
	 */
	public static Konstruktor[] fetchAll (ClassInfo cinfo)
	{
		LinkedList<Konstruktor> list = new LinkedList<Konstruktor>();
		
		for (Constructor con : cinfo.constructors())
		{
			if (!con.isAnnotationPresent(cexport.class))
				 continue;
			list.add(new Konstruktor(con));
		}

		if (list.size() == 0)
			list.add(new Konstruktor(cinfo.name()));

		Konstruktor[] ret = new Konstruktor[list.size()];
		for (int i=0;i<ret.length;i++)
			ret[i] = list.get(i);

		return ret;
	}
}
