package DotClass;

import ReflWrap.*;
import java.io.*;
import java.util.*;

/** DotClass - generate class diagramms from source code */
public class Main
{
	/** Resolve a symbol
	 * @param list List of classes
	 * @param klasse Class
	 */
	public static void resolveSymbol (ClassList list, Class klasse)
	{	
		if (klasse==null)
			return;
		
		if (klasse.getName().equals("java.lang.Object"))
		{
			list.add(new ClassInfo(klasse));
			return;
		}
			
		ClassInfo cinfo = new ClassInfo(klasse);
		list.add(cinfo);
		
		// Resolve superclass
		resolveSymbol(list, cinfo.sclass());
		
		// Resolve interfaces
		for (int j=0;j<cinfo.interfaces().length;j++)
			resolveSymbol(list, cinfo.interfaces()[j]);
	}
	
	public static void main (String[] args)
	{
		if (args.length<2)
		{
			System.out.println("DotClass - generate class hierarchy graphs.");
			System.out.println("\nUsage: java Tools.DotClass [directory] "+
					"[package]");
			System.exit(1);
		}

		// TODO: hack
		CTL.Env.newLogger("/dev/null", false);
		
		try
		{
			File dir = new File(args[0]+"/"+args[1]);
			File[] classes = dir.listFiles();
			ClassList list = new ClassList();

			if (classes==null)
			{
				System.out.println("No classes found.");
				System.exit(2);
			}
			
			for (int i=0;i<classes.length;i++)
			{
				if ((classes[i].getName().equals("CVS"))||
					(classes[i].getName().equals(".cvsignore"))||
					(classes[i].getName().equals(".svn")))
					continue;
				if (classes[i].isDirectory())
					continue;
					
				StringTokenizer tok = new StringTokenizer(classes[i].
						getName(), ".");
				resolveSymbol(list, 
						Class.forName(args[1]+"."+tok.nextToken()));
			}

			FileWriter ofile = new FileWriter(args[1]+".dot");

			ofile.write("digraph "+args[1]+"\n{\n");
			
			ListIterator iter = list.iter();
			while (iter.hasNext())
			{
				ClassListEntry entry = (ClassListEntry)iter.next();
				entry.resolve(list);

				//System.out.println(entry);

				String decl = null;
				if (entry.isIFace())
					decl = DefaultStyle.declareIFace(entry);
				else
					decl = DefaultStyle.declareClass(entry);

				ofile.write(decl);
			}

			ofile.write("\n");

			iter = list.iter();
			while (iter.hasNext())
			{
				ClassListEntry entry = (ClassListEntry)iter.next();

				if (!entry.isIFace())
					ofile.write(DefaultStyle.implement(entry.sclass(), 
								entry.dot_name()));
				
				for (int i=0;i<entry.numIFaces();i++)
					ofile.write(DefaultStyle.iface(entry.printIFace(i), 
								entry.dot_name()));
			}

			ofile.write("\n"+DefaultStyle.copyleft()+"}\n");
			ofile.close();
		}
		catch (Exception e)
		{
			System.out.println(e);
		}
	}
}
