package Hitman.HTTP;

import Hitman.*;
import java.io.*;
import java.util.*;

/** HTTP header */
public class Header
{
	private static String[] udef = new String[6];
	
	protected String proto = "HTTP/1.0";
	protected short retcode = -1;
	protected String req = null;
	protected String repl = null;
	protected long len = -1;
	protected Date date = null;
	protected Date last_mod = null;
	protected HashMap<String,String> usrdef = null;

	static
	{
		udef[0] = "Server";
		udef[1] = "ETag";
		udef[2] = "Accept-Ranges";
		udef[3] = "Connection";
		udef[4] = "Content-Type";
		udef[5] = "User-Agent";
	}

	public Header (String req, String path, long len)
	{
		this.req = req;
		this.repl = path;
		this.len = len;
		
		usrdef = new HashMap<String,String>();
		usrdef.put(udef[5], Request.uagent);
	}

	public Header (short code)
	{
		retcode = code;
		repl = getReply(retcode);
		date = new Date();

		usrdef = new HashMap<String,String>();
		usrdef.put(udef[0], "Hitman server v0.1");
		usrdef.put(udef[3], "close");
	}
	
	public Header (short code, String type, String cnt)
	{
		this(code);
		len = cnt.length();
		last_mod = new Date();
		usrdef.put(udef[4], type);
	}

	public Header (String type, String cnt)
	{
		this((short)200, type, cnt);
	}
	
	public Header (BufferedReader buf)
	{
		boolean read = true;
		usrdef = new HashMap<String,String>();
		
		try
		{
			String msg = buf.readLine();
			StringTokenizer str = new StringTokenizer(msg, " ");
			
			if (str.countTokens()<3)
			{
				System.err.println("Weird HTTP reply.");
				retcode = 417;
				repl = "Expectation Failed";
			}
			else
			{
				proto = str.nextToken();

				if (proto.equals("GET") ||
					proto.equals("POST"))
				{
					String moo = proto;
					repl = str.nextToken();
					proto = str.nextToken();
					req = moo;
				}
				else
				{
					retcode = Short.parseShort(str.nextToken());
					StringBuffer b = new StringBuffer();
					while (str.hasMoreTokens())
						b.append(str.nextToken()+" ");
					repl = b.toString();
				}
			}

			//System.out.println(retcode);

			if (retcode>=400)
			{
				System.out.println("Error occured: "+retcode+" "+repl);
				return;
			}
			
			while (read)
			{
				String line = buf.readLine();
				if (line.equals(""))
					read=false;
				else
				{
					StringTokenizer tok = new StringTokenizer(line, ":");
					if (tok.countTokens()<2)
						continue;
					String key = tok.nextToken();
					String value = line.substring(key.length()+2, 
						line.length());

					for (int i=0;i<udef.length;i++)
						if (key.equalsIgnoreCase(udef[i]))
							usrdef.put(udef[i], value);

					if (key.equalsIgnoreCase("Content-length"))
						len = Long.parseLong(value);
					if (key.equalsIgnoreCase("Date"))
						date = Dating.parse(value);
					if (key.equalsIgnoreCase("Last-Modified"))
						last_mod = Dating.parse(value);
					if (key.equalsIgnoreCase("Transfer-Encoding"))
						if (value.equalsIgnoreCase("chunked"))
							len = -2; // Chunked transfer
				}
			}
		} 
		catch (Exception e) 
		{
			e.printStackTrace();
		}
	}

	public String get (String key)
	{
		return usrdef.get(key);
	}

	private String userdef ()
	{
		StringBuffer buf = new StringBuffer();
		Set<Map.Entry<String,String>> set = usrdef.entrySet();
		for (Map.Entry<String,String> m : set)
			buf.append(m.getKey()+": "+m.getValue()+"\n");
		return buf.toString();
	}	

	public String toString ()
	{
		StringBuffer buf = new StringBuffer();
		if (req!=null)
			buf.append(req+" "+repl+" "+proto+"\n");
		buf.append(userdef());
		return buf.toString();
	}

	public void writeTo (BufferedWriter out) throws IOException
	{
		StringBuffer buf = new StringBuffer();
		
		if (req==null)
		{
			buf.append(proto+" "+retcode+" "+repl+"\r\n");
			buf.append("Date: "+date+"\n");
			if (last_mod!=null)
				buf.append("Last-Modified: "+last_mod+"\n");
		}
		else
			buf.append(req+" "+repl+" "+proto+"\r\n");
			
		if (len!=-1)
			buf.append("Content-Length: "+len+"\n");
		buf.append(userdef()+"\n\n");
		out.write(buf.toString(), 0, buf.toString().length());
	}

	public String getReply (short code)
	{
		switch (code)
		{
			case 200:
				return "OK";	
			case 404:
				return "Not Found";
			case 500:
				return "Internal Server Error";
		}
		
		return null;
	}
	
	public void addHeader (String key, String value)
	{
		usrdef.put(key, value);
	}
}
