package Hitman.HTTP;

import Hitman.*;
import java.io.*;
import java.net.*;
import java.util.*;

/** Simple HTTP daemon */
public class Server
{
	/** Document root */
	private static String www_root = "www";
	
	/** Private listen thread */
	private static class ListenThread extends Thread
	{
		private ServerSocket ssock = null;
			
		public ListenThread (ServerSocket ssock)
		{
			this.ssock = ssock;
		}
			
		public void run ()
		{
			Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
			
			try
			{
				//while (true)
				{
					Socket sock = ssock.accept();
					Request req = new Request(sock);
					BufferedWriter write = (BufferedWriter)req.recv();
					
					if (req.head.repl.equals("/"))
						Server.reply(write, "text/plain", "java httpd");
					else
					{
						File file = new File(Server.www_root, req.head.repl);
						if (!file.exists())
							Server.error(write, (short)404);
						else
							Server.reply(write, file);
					}

					sock.close();
				}
			}
			catch (IOException e) 
			{
				e.printStackTrace();
			}
		}
	}
	
	/** Handle errors
	 * @param out Output stream
	 * @param errcode Error code
	 */
	private static void error (BufferedWriter out, short errcode) 
		throws IOException
	{
		Header header = null;
		String repl = null;
		
		switch (errcode)
		{
			case 404:
				StringBuffer buf = new StringBuffer();
				buf.append("<html><head><title>Mogguh!</title></head>");
				buf.append("<body>Document not found.</body></html>");
				repl = buf.toString();
				header = new Header(errcode, "text/html", repl);
				break;
			default:
				header = new Header((short)500);
				break;
		}

		header.writeTo(out);
		if (repl!=null)
			out.write(repl, 0, repl.length());
		out.flush();
	}

	/** Reply to a POST request
	 * @param out Output stream
	 * @param cntType Content type
	 * @param cnt Content
	 */
	private static void reply (BufferedWriter out, String cntType, String cnt)
		throws IOException
	{
		Header head = new Header(cntType, cnt);
		head.writeTo(out);	
		out.write(cnt, 0, cnt.length());
		out.flush();
	}

	/** Reply to a GET request
	 * @param out Output stream
	 * @param file Requested file
	 */
	private static void reply (BufferedWriter out, File file) 
		throws IOException
	{
		long len = file.length();
		if (len>Integer.MAX_VALUE)
			System.err.println("Hitman does not support large files.");

		if (IOUtil.getExt(file).equals("cgi"))
		{
			handle_cgi(out, file);
			return;
		}

		byte[] buf = new byte[(int)len];
		FileInputStream in = new FileInputStream(file);
		in.read(buf);
		in.close();
		
		reply(out, IOUtil.getFileType(file), new String(buf));	
	}

	/** Handle a CGI call request
	 * @param out Output stream
	 * @param cgi CGI to call
	 */
	private static void handle_cgi (BufferedWriter out, File cgi) 
		throws IOException
	{
		String cmd = "/usr/bin/env -i HOME=/dev/null PATH=/bin:/usr/bin "+
			cgi.getAbsolutePath();
		
		String res = IOUtil.exec(cmd);
		StringTokenizer tok = new StringTokenizer(res, "\n\n");
		String typ = tok.nextToken();
		StringTokenizer tok2 = new StringTokenizer(typ, ":");
		String moo = tok2.nextToken();
		String type = typ.substring(moo.length()+2, typ.length());
		
		StringBuffer cnt = new StringBuffer();
		while (tok.hasMoreTokens())
			cnt.append(tok.nextToken()+"\n");

		reply(out, type, cnt.toString());
	}

	/** Main program */
	public static void main (String[] args)
	{
		try
		{
			ServerSocket ssock = new ServerSocket(0, 1);
			System.out.println("Listening on "+ssock.getLocalPort());
			ListenThread thr = new ListenThread(ssock);
			thr.start();
			IOUtil.exec("java -cp src Hitman.HTTP.Client "+
				"http://localhost:"+ssock.getLocalPort()+"/");
			thr.join();
			ssock.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}
}
