package NCLib;

import java.io.*;
import java.util.*;
import java.util.regex.*;

/** Implementation of a simple file templating mechanism */
public class Template
{
	/** Contents of the input file */
	private String input;
	/** Map of the replaceables */
	private HashMap<String,String> map;

	/** 
	 * Default constructor
	 * @param fname Filename of the input file 
	 */
	public Template (String fname)
	{
		map = new HashMap<String,String>();
		input = Files.readAll(fname);
		Pattern p = Pattern.compile("%.*\\%");
		Matcher m = p.matcher(input);
		while (m.find())
		{
			String tmp = m.group();
			tmp = tmp.substring(1, tmp.length()-1);
			map.put(tmp, "");
		}
	}

	/**
	 * Returns a set of all keywords found in the input file
	 * @return The set, motherfucker!
	 */
	public Set<String> fields ()
	{
		return map.keySet();
	}

	/** 
	 * Set a string as replacement for a specific keyword
	 * @param key The keyword
	 * @param value The replacement string (oh, the surprise!)
	 * @return True if the keyword was in the set of keywords found in the input
	 * file, false otherwise (and no replacing then, of course)
	 */
	public boolean set (String key, String value)
	{
		if (!map.containsKey(key))
			return false;
		map.put(key, value);
		return true;
	}

	/** 
	 * Set replacements for multiple keywords at once
	 * @param repl hashmap of key, value
	 */
	public void setAll (HashMap<String,String> repl)
	{
		for (String key : repl.keySet())
			set(key, repl.get(key));
	}

	/** 
	 * Retrieve a string representation of this object
	 * @return The string
	 */
	public String toString ()
	{
		String ret = input;
		for (String key : fields())
		{
			String value = map.get(key);
			Pattern p = Pattern.compile("%"+key+"%");
			Matcher m = p.matcher(ret);
			ret = m.replaceAll(value);
		}
		return ret;
	}

	/** 
	 * Write the resulting file
	 * @param fname Filename of the destination file
	 */
	public void write (String fname)
	{
		Files.writeAll(fname, toString());
	}

	/** Test routine */
	public static void main (String[] args)
	{
		Template tmpl = new Template("data/test.tmpl");
		tmpl.set("name", "Frankie");
		tmpl.set("name2", "Ocle");
		System.out.print(tmpl);
	}
}
